# Część 9: Podstawy wektorów 


**Wektory** to podstawowe narzędzie w grafice komputerowej 2D i 3D. Geometrycznie wektor wyobrażamy sobie jako strzałkę zaczepioną w przestrzeni – ma on **kierunek** i **magnitude (długość)**, ale nie ma ustalonego punktu zaczepienia. W odróżnieniu od **punktu** (określonego współrzędnymi w przestrzeni), wektor reprezentuje przesunięcie lub ruch o pewną wartość wzdłuż osi X, Y, Z. W praktyce punkt i wektor zapisujemy podobnie (np. `(x, y, z)`), jednak interpretujemy je inaczej: punkt to konkretny *miejsce* w przestrzeni, a wektor to *odległość i kierunek*.

W grafice wektory są wykorzystywane wszędzie: do określania położenia wierzchołków (jako odległości od początku układu współrzędnych), do przesuwania obiektów na ekranie, do obliczania kierunków oświetlenia i wielu innych zadań. Operacje na wektorach to m.in. dodawanie (łączenie przesunięć), skalowanie (wydłużanie/skracanie wektora) oraz iloczyn skalarny i wektorowy, które pozwalają obliczać odpowiednio kąty między wektorami i wektor prostopadły. Dwie często używane wielkości wektorowe to **długość wektora** (mówi, jak daleko sięga wektor) oraz **kąt między dwoma wektorami** (określa, jak bardzo różnią się kierunki wektorów).

W tym laboratorium skupimy się na podstawowych operacjach wektorowych. Najpierw zobaczymy różnicę między punktem a wektorem, następnie użyjemy wektorów do przesuwania obiektu w scenie 3D. Dodamy do sceny siatkę (grid) ułatwiającą obserwację ruchu oraz nauczymy się mierzyć przesunięcia. Następnie zbadamy skalowanie wektorów – przekonamy się, że mnożąc wektor przez skalar zmieniamy jego długość, ale nie kierunek (chyba że użyjemy liczby ujemnej, co odwróci wektor). Na końcu obliczymy długość wektora, stosując twierdzenie Pitagorasa w ujęciu trójwymiarowym.

### Ruch obiektu za pomocą wektora

1. Utwórz nowy folder. Skopiuj do niego aktualne wersje wszystkich plików, czyli m.in. `Button.py`, `Cube.py`, `Mesh3D.py`, `Object.py`, `Settings.py`, `Transform.py`, `Utils.py` oraz stworzony wcześniej `LoadMesh.py` (nawet jeśli nie będziemy tu ładować nowego modelu, część tych plików jest potrzebna do działania silnika). Ponadto upewnij się, że masz skopiowany plik z kodem głównym z końca części 8 – prawdopodobnie nosił on nazwę `DisplayTeapot.py` (lub podobną). 

2. Utwórz nowy skrypt Pythona o nazwie **Vectors.py**. Będzie to główny program dla tej części. Wypełnij go kodem z poprzedniego etapu (cz. 7, plik `AddingButtons.py`), **ale usuń fragmenty odpowiedzialne za tworzenie przycisku**. Innymi słowy, nasz program powinien otwierać okno graficzne, wyświetlać animowaną scenę 3D z obiektem (sześcianem) i umożliwiać sterowanie nim klawiaturą – ale nie chcemy teraz żadnych przycisków GUI na ekranie. Poniżej wskazano fragment kodu z części 7, który należy **usunąć** z `Vectors.py` (był odpowiedzialny za przycisk):

   ```python
   def button_click():
       print("Hello Button")
   
   white = pygame.Color(255, 255, 255)
   green = pygame.Color(0, 255, 0)
   blue = pygame.Color(0, 0, 255)
   button1 = Object("Button")
   button1.add_component(Button(screen, (0, 0), 100, 50, green, white, blue, button_click))
   objects_2d.append(button1)
   ```

   Po usunięciu powyższych linii, program w `Vectors.py` powinien tworzyć scenę z ruchomym sześcianem tak jak wcześniej, tylko bez przycisku. Uruchom tymczasowo `Vectors.py`, aby upewnić się, że działa (powinno otworzyć okno z obracającym się sześcianem, którym można sterować strzałkami – zakładając, że takie sterowanie zostało zaimplementowane we wcześniejszych częściach kursu – lub po prostu statycznym, jeśli ruch nie był wcześniej dodany; najważniejsze, że scena się renderuje).

3. Aby ułatwić obserwację ruchu obiektu w przestrzeni, dodamy **siatkę (grid)** na podłodze sceny. Siatka pozwoli nam wizualnie mierzyć przesunięcia. Utwórz nowy skrypt Pythona o nazwie **Grid.py** i dodaj do niego następujący kod:

   ```python
   from OpenGL.GL import *
   
   class Grid():
       def __init__(self, interval, halfsize, colour):
           self.interval = interval
           self.halfsize = halfsize
           self.colour = colour
   
       def draw(self):
           glColor3fv(self.colour)
           glBegin(GL_LINES)
           for x in range(-self.halfsize, self.halfsize):
               for y in range(-self.halfsize, self.halfsize):
                   glVertex3fv((x * self.interval,       y * self.interval - 10,  0))
                   glVertex3fv((x * self.interval,       y * self.interval + 500, 0))
                   glVertex3fv((y * self.interval - 10,  x * self.interval,       0))
                   glVertex3fv((y * self.interval + 500, x * self.interval,       0))
           glEnd()
   ```

   Klasa `Grid` definiuje obiekt siatki. Jej konstruktor przyjmuje:
   - `interval` – odstęp (odległość) między kolejnymi liniami siatki,
   - `halfsize` – połowę liczby linii w obu kierunkach (czyli siatka rozciąga się od `-halfsize` do `halfsize` w obu osiach),
   - `colour` – kolor linii (w formie krotki RGB np. `(0, 0, 255)` dla niebieskiego).

   Metoda `draw()` rysuje siatkę za pomocą wielu linii. Ustawiamy kolor linii przez `glColor3fv(self.colour)`. Następnie `glBegin(GL_LINES)` oznacza, że podamy listę wierzchołków, które OpenGL połączy parami jako odcinki. W podwójnej pętli `for` generujemy współrzędne punktów tak, by narysować serię linii równoległych do osi X i Z oraz do osi Y i Z:
   - Pierwsze dwa wywołania `glVertex3fv` wewnątrz pętli rysują linię pionową (wzdłuż osi Y) w stałej pozycji X = `x * interval` i Z = 0, rozciągającą się od Y = `-10` do Y = `+500`. Te wartości Y wybrano tak, by linie były wystarczająco długie i widoczne (tworzą siatkę przecinającą przestrzeń w pionie).
   - Kolejne dwa wywołania `glVertex3fv` rysują linię poziomą (wzdłuż osi X) w stałej pozycji Y = `x * interval` i Z = 0, rozciągającą się analogicznie.
   
   W efekcie powstaje płaska siatka znajdująca się na płaszczyźnie X-Y (na wysokości Z = 0). Kolor siatki ustawiamy przy tworzeniu obiektu (np. niebieski).

4. Zaktualizuj plik **Vectors.py**, aby wykorzystać klasę `Grid`. Importuj klasę (`from Grid import *`) i utwórz instancję siatki, a następnie dodaj ją do sceny. W tym celu odszukaj miejsce, gdzie tworzony jest obiekt `cube` i dodawany do listy `objects_3d`. Poniżej dodaj kod tworzący obiekt siatki:

   ```python
   from Grid import *
   from Object import *
   ...
   cube.add_component(Cube(GL_POLYGON, "images/wall.tif"))
   objects_3d.append(cube)
   
   grid = Object("Grid")
   grid.add_component(Transform((0, 0, -5)))
   grid.add_component(Grid(0.5, 8, (0, 0, 255)))
   objects_3d.append(grid)
   clock = pygame.time.Clock()
   ```

   Wyjaśnienie:
   - Importujemy moduł `Grid` zawierający naszą klasę.
   - Tworzymy nowy obiekt sceny `grid` typu `Object` (analogicznie jak `cube`).
   - Dodajemy do niego komponent `Transform((0, 0, -5))`, który przesuwa całą siatkę o `(0, 0, -5)`. Zakładamy tu, że początkowo nasz sześcian `cube` również był odsunięty o `(0, 0, -5)` (tak jak w poprzedniej części kursu). To odsunięcie w osi Z zapewnia, że zarówno sześcian, jak i siatka, znajdują się nieco dalej od kamery, dzięki czemu widzimy je w całości.
   - Dodajemy komponent `Grid(0.5, 8, (0, 0, 255))` – tworzymy siatkę z odstępem 0,5 jednostki, rozciągającą się 8 jednostek w każdą stronę od środka, w kolorze niebieskim.
   - Dodajemy obiekt `grid` do listy `objects_3d`, by był renderowany.
   - (W powyższym kodzie pokazano również linię dodającą komponent `Cube` do `cube`. Upewnij się, że używasz odpowiedniej ścieżki do pliku tekstury, jeśli w Twoim projekcie pliki są w innej lokalizacji lub używasz innej tekstury).

   Po dodaniu powyższego kodu, **przesuń również obiekt `cube` o taką samą wartość Z** (czyli upewnij się, że `cube` też ma `Transform((0, 0, -5))`). Dzięki temu sześcian i siatka będą na tej samej płaszczyźnie Z. Jeśli sześcian był wcześniej na `(0,0,0)`, a siatka na `(0,0,-5)`, to sześcian znalazłby się nad siatką (bliżej kamery). Dlatego istotne jest spójne przesunięcie.

5. Zaktualizuj klasę **Object** w pliku `Object.py`, aby obsługiwała rysowanie siatki. Do tej pory metoda `Object.update()` zapewniała rysowanie komponentów takich jak `Transform`, `Mesh3D` czy `Button`. Teraz dodamy obsługę typu `Grid`. W funkcji `update(self, events=None)` klasy `Object` znajdź pętlę przechodzącą po komponentach (`for c in self.components:`) i dopisz kolejną gałąź `elif` sprawdzającą instancję `Grid`. Po zmianach fragment funkcji `update` powinien wyglądać np. tak:

   ```python
   def update(self, events=None):
       glPushMatrix()
       for c in self.components:
           if isinstance(c, Transform):
               pos = c.get_position()
               glTranslatef(pos.x, pos.y, pos.z)
           elif isinstance(c, Mesh3D):
               c.draw()
           elif isinstance(c, Grid):
               c.draw()
           elif isinstance(c, Button):
               c.draw(events)
       glPopMatrix()
   ```

   Tutaj dodaliśmy blok:
   ```python
   elif isinstance(c, Grid):
       c.draw()
   ```
   co spowoduje, że jeżeli obiekt posiada komponent typu `Grid`, to przy aktualizacji (rysowaniu obiektu) wywołana zostanie metoda `draw()` naszej siatki. Dzięki temu siatka będzie faktycznie rysowana na ekranie, analogicznie jak inne komponenty.

6. Dodajmy teraz funkcjonalność pozwalającą przesuwać sześcian za pomocą wektora. W pliku **Transform.py** (klasa transformacji obiektu) dodaj nową metodę `move()`, która będzie przyjmowała wektor przesunięcia i odpowiednio zmieniała pozycję obiektu. Metoda powinna dodawać wektor przesunięcia do aktualnej pozycji. Możesz dodać następujący kod do klasy `Transform`:

   ```python
   def move(self, amount: pygame.math.Vector3):
       self.position = pygame.math.Vector3(
           self.position.x + amount.x,
           self.position.y + amount.y,
           self.position.z + amount.z
       )
   ```

   Metoda `move` przyjmuje obiekt typu `pygame.math.Vector3` (wektor 3D z biblioteki Pygame) i zwiększa składowe `position` o odpowiednie wartości. `self.position` jest zapewne przechowywane również jako `pygame.math.Vector3`, więc można by napisać prościej `self.position += amount`, ale powyższy zapis explicite dodaje każdą współrzędną. Po wywołaniu `move`, obiekt zostanie przesunięty o zadany wektor.

7. Wykorzystaj nową metodę `move` w programie **Vectors.py**. Chcemy, aby wciśnięcie klawisza spacji przesunęło sześcian w prawo (wzdłuż osi X). W tym celu w pętli zdarzeń Pygame (obsługującej naciśnięcia klawiszy) dodaj obsługę klawisza **SPACJA**. Odszukaj fragment kodu w `Vectors.py` podobny do:

   ```python
   while not done:
       events = pygame.event.get()
       for event in events:
           if event.type == pygame.QUIT:
               done = True
           if event.type == KEYDOWN:
               # ... obsługa innych klawiszy ...
   ```

   i wewnątrz warunku `if event.type == KEYDOWN:` dodaj kolejny podwarunek dla spacji:

   ```python
   if event.key == K_SPACE:
       trans.move(pygame.Vector3(0.5, 0, 0))
   ```

   Upewnij się, że wcześniej w kodzie masz referencję do transformacji `cube` – np. tuż po utworzeniu `cube` możesz pobrać komponent transformacji: `trans: Transform = cube.get_component(Transform)`. Takie coś było już prawdopodobnie używane dla sterowania strzałkami. Jeżeli nie, możesz na początku programu (po utworzeniu `cube` i dodaniu komponentu `Transform`) przypisać:

   ```python
   trans = cube.get_component(Transform)
   ```

   co pozwoli łatwo odwoływać się do transformacji sześcianu.

   W efekcie, gdy w trakcie działania programu wciśniesz spację, zostanie wywołane `trans.move(pygame.Vector3(0.5, 0, 0))`, które przesunie sześcian o `0.5` jednostki w osi X (w prawo).

8. Uruchom program **Vectors.py**. Zaobserwuj początkowe położenie sześcianu na siatce, a następnie wciśnij kilkukrotnie klawisz spacji. Każde naciśnięcie powinno przesunąć sześcian o 0.5 w prawo. Ponieważ linie siatki są rozmieszczone co 0.5 jednostki, ruch sześcianu będzie odpowiadał przemieszczeniu się o jedną "kratkę" siatki w prawo. Jeśli sześcian startował na środku (0,0,-5), po jednym wciśnięciu spacji powinien znaleźć się nad linią siatki x=0.5 (czyli na współrzędnej (0.5, 0, -5)), a po dwóch wciśnięciach – na (1.0, 0, -5) i tak dalej.

   Spróbuj też użyć strzałek (jeśli były zaimplementowane wcześniej do ruchu) – powinny nadal przesuwać sześcian w lewo/prawo/góra/dół, a siatka pozwoli łatwo zobaczyć o ile jednostek się przesunął.

   Widzimy tu wyraźnie, że pozycja sześcianu jest punktem (np. (0,0,-5)), natomiast do jego zmiany używamy **wektora** (np. (0.5,0,0) – ruch wzdłuż osi X). Za każdym razem dodajemy ten sam wektor do pozycji, co skutkuje przesunięciem. Wektor (0.5,0,0) sam w sobie nie ma stałego miejsca początkowego – jest jak "instrukcja" przesunięcia obiektu o pół jednostki w prawo. Dodając go kolejno do pozycji, otrzymujemy kolejne punkty położenia obiektu.

### Skalowanie wektora – ruch drugiego obiektu

9. **Dodaj drugi obiekt (drugi sześcian) do sceny**, aby zbadać efekt skalowania wektora. W pliku **Vectors.py** utwórz kolejny obiekt `cube2`. Możesz to zrobić tuż po utworzeniu i skonfigurowaniu pierwszego `cube`. Przykładowy kod:

   ```python
   cube.add_component(Cube(GL_POLYGON, "Chapter_Four/images/wall.tif"))
   cube2 = Object("Cube")
   cube2.add_component(Transform((0, 1, -5)))
   cube2.add_component(Cube(GL_POLYGON, "Chapter_Four/images/brick.tif"))
   objects_3d.append(cube)
   objects_3d.append(cube2)
   ```

   Wyjaśnienie:
   - Tworzymy `cube2` analogicznie do `cube`.
   - Przesuwamy `cube2` o `(0, 1, -5)` – czyli umieszczamy go 1 jednostkę *nad* pierwszym sześcianem (w osi Y) i tak samo 5 jednostek w głąb ekranu.
   - Dodajemy komponent `Cube` z jakąś teksturą (tutaj użyto innego pliku `"brick.tif"` jako przykładowej tekstury, możesz użyć dowolnej innej, by łatwo odróżnić oba obiekty).
   - Dodajemy oba obiekty do listy `objects_3d`. (Upewnij się, że lista była wcześniej pusta lub czyścisz ją przed dodaniem, aby nie dodać ponownie pierwszego cube – w naszym kodzie powyżej dodaliśmy cube do listy przed utworzeniem cube2, co jest poprawne.)

   Po tej zmianie, uruchom program aby sprawdzić, czy na ekranie pojawiają się dwa sześciany – jeden pod drugim. Drugi sześcian powinien być widoczny powyżej pierwszego (o ile nic go nie zasłania).

10. Zmodyfikuj obsługę klawisza spacji, aby poruszała oba sześciany jednocześnie, ale z różnym skalowaniem wektora ruchu. Wykorzystamy to, by pokazać, że mnożenie wektora przez skalar zmienia jego długość, ale nie kierunek. Znajdź w kodzie miejsce, gdzie obsługujemy `K_SPACE` i dodaj przesunięcie także dla `cube2`. Przykładowo:

    ```python
    if event.key == K_SPACE:
        trans.move(pygame.Vector3(1, 1, 0))
        trans2.move(pygame.Vector3(1, 1, 0) * 2)
    ```

    Wyjaśnienie:
    - Dla pierwszego sześcianu (`trans`) używamy wektora `(1, 1, 0)`. To oznacza przesunięcie o 1 jednostkę w prawo (X) i 1 jednostkę w górę (Y).
    - Dla drugiego sześcianu (`trans2`) używamy **tego samego wektora**, ale pomnożonego przez 2, czyli `pygame.Vector3(1, 1, 0) * 2`, co daje wektor `(2, 2, 0)`.
    - W rezultacie, po wciśnięciu spacji pierwszy sześcian przesunie się o (1,1,0), a drugi o (2,2,0).

    Uruchom aplikację i naciśnij spację. Obserwuj ruch obu sześcianów. Powinny przesunąć się w tym samym kierunku (ukośnie w prawo i do góry), ale drugi sześcian pokona dwa razy większy dystans. Jeśli pierwszy sześcian przesunął się o 1 kratkę w prawo i 1 do góry, to drugi przesunie się o 2 kratki w prawo i 2 do góry. Wektory przesunięcia były równoległe (wskazywały ten sam kierunek), różniły się tylko długością.

    Zwróć uwagę, że klasa `pygame.Vector3` umożliwia łatwe skalowanie wektorów przez zwykłe mnożenie (`* 2`).

11. W powyższym przykładzie pokazaliśmy, że **skalowanie wektora nie zmienia jego kierunku**, o ile mnożnik jest dodatni. Wspomnieliśmy jednak, że mnożnik ujemny może odwrócić wektor. Spróbujmy to zaobserwować. Zmień w kodzie mnożnik dla drugiego sześcianu na wartość ujemną, np. -2:

    ```python
    trans2.move(pygame.Vector3(1, 1, 0) * -2)
    ```

    Teraz po wciśnięciu spacji pierwszy sześcian nadal będzie poruszał się o (1,1,0) w prawo i w górę, natomiast drugi sześcian otrzyma wektor `(-2,-2,0)` (bo `(1,1,0) * -2` daje `(-2,-2,0)`). Oznacza to ruch o 2 w lewo i 2 w dół. Uruchom program ponownie, wciśnij spację i zobacz, że drugi sześcian porusza się w dokładnie przeciwnym kierunku do pierwszego.

    Otrzymujemy w ten sposób potwierdzenie: pomnożenie wektora przez liczbę ujemną **odwraca jego kierunek** (o 180 stopni), jednocześnie skalując długość według wartości bezwzględnej mnożnika.

12. **Obliczanie długości wektora (magnitude)**. Wektor możemy traktować jako odcinek w przestrzeni zaczepiony w początku układu współrzędnych. Długość wektora w 3D można obliczyć na podstawie jego składowych korzystając z uogólnionego twierdzenia Pitagorasa: 

    \[
    |\mathbf{v}| = \sqrt{v_x^2 + v_y^2 + v_z^2}.
    \]

    Pygame oferuje metodę `Vector3.length()` zwracającą długość wektora. Możesz poeksperymentować w kodzie, np. obliczając długości niektórych wektorów i wypisując je do konsoli przy starcie programu:
    ```python
    v = pygame.Vector3(2, 3, 4)
    print("Długość wektora (2,3,4) =", v.length())
    ```
    Powinno to wyświetlić wartość równą $\sqrt{2^2 + 3^2 + 4^2} = \sqrt{4 + 9 + 16} = \sqrt{29} \approx 5.39$. 
    
    Spróbuj obliczyć i sprawdzić w ten sposób kilka wektorów (np. `(1,0,0)` ma długość 1, `(1,1,0)` ma długość $\sqrt{2}\approx1.414`, `(0,3,4)` ma długość 5 itp.). Zrozumienie tej koncepcji będzie przydatne w dalszych zadaniach, gdy będziemy np. normalizować wektory czy wyznaczać odległości między punktami.

## Wyjaśnienie kodu

W powyższych krokach rozbudowaliśmy nasz silnik graficzny o możliwość korzystania z wektorów:

- **Klasa Grid** rysuje siatkę linii, co pozwala wizualnie ocenić przemieszczenia obiektów. Każda iteracja wewnętrznej pętli `for` dodaje dwa punkty (`glVertex3fv`) definiujące odcinek linii. Dla danych `x` i `y` rysujemy:
  - Linię równoległą do osi Y (pionową) na pozycji X = `x * interval`, rozciągniętą od Y = `-10` do Y = `+500` (te wartości wybrano arbitralnie, by linie były dostatecznie długie – faktycznie rysowana siatka będzie większa niż obszar widoczny, co nie szkodzi).
  - Linię równoległą do osi X (poziomą) na pozycji Y = `x * interval`, rozciągniętą od X = `-10` do X = `+500`.
  - W rezultacie powstaje zestaw linii tworzących kwadratową siatkę. Kolor ustawiony przez `glColor3fv` spowoduje narysowanie wszystkich kolejnych odcinków w danym kolorze (np. niebieskim).
  - Zakończenie `glEnd()` zamyka definicję wszystkich odcinków linii.
  - Ta implementacja rysuje więcej linii niż potrzeba (prawdopodobnie aż do współrzędnych 500, co jest daleko poza typowym zasięgiem widzenia), ale nie wpływa to negatywnie na zrozumienie działania – ewentualnie można by ograniczyć długość linii. Ważne jest, że linie są rozmieszczone co `interval` jednostek i tworzą wzór kratki, co zostało osiągnięte.

- **Modyfikacja Object.update()**: Dodając warunek `elif isinstance(c, Grid): c.draw()`, sprawiliśmy, że obiekty zawierające komponent siatki będą je rysować. Bez tego warunku nasz obiekt `grid` istniałby w scenie, ale jego `Grid` nigdy by się nie narysował, bo `update()` by go ignorowało (nie podpada pod inne przypadki). Teraz jest obsługiwany analogicznie do `Mesh3D` czy `Button`.

- **Metoda Transform.move()**: Umożliwia łatwe przemieszczanie obiektu. Korzystamy z tego, że klasa `pygame.math.Vector3` pozwala reprezentować pozycję i wykonywać na niej operacje wektorowe. Dodanie wektorów pozycji i przesunięcia to istota ruchu: nowa pozycja = stara pozycja + wektor_przesunięcia. Dzięki temu możemy sterować ruchem obiektu, przekazując różne wektory do tej metody.

- **Dodanie drugiego obiektu (cube2)**: Pokazaliśmy, jak mieć dwie niezależne instancje w scenie. Obie mają własne komponenty `Transform` i `Cube`, dzięki czemu mogą być przemieszczane i renderowane niezależnie. Umieszczenie `cube2` nieco wyżej (`(0,1,-5)`) zapewniło, że obiekty nie nachodzą na siebie i widać je osobno.

- **Skalowanie wektora ruchu**: W kodzie obsługi spacji zastosowaliśmy dwa różne wektory:
  - Dla `cube`: `(1,1,0)`
  - Dla `cube2`: `(1,1,0) * 2` czyli `(2,2,0)`.
  Oba te wektory mają ten sam kierunek (diagonalnie w płaszczyźnie X-Y), różnią się tylko długością (drugi jest dwa razy dłuższy). W wyniku, wciśnięcie spacji przesunęło obydwa sześciany równolegle – to samo nachylenie trajektorii – ale jeden przemieścił się o większy dystans. Zobaczyliśmy, że mnożąc wektor przez skalary, możemy łatwo kontrolować odległość ruchu.
  
  Kiedy zmieniliśmy mnożnik na -2, drugi sześcian otrzymał wektor `(-2,-2,0)`. Kierunek tego wektora jest przeciwny do `(1,1,0)` (wektor `(1,1,0)` odwrócony o 180° daje `(-1,-1,0)`, a skalowanie do `(-2,-2,0)` tylko wydłuża go dwukrotnie). Eksperyment ten potwierdził teorię: *dodatni* mnożnik zachowuje kierunek wektora (ewentualnie wektor zerowy, gdy mnożnik=0, nie ma kierunku), *ujemny* odwraca zwrot wektora.

- **Długość wektora**: Wspomnieliśmy, jak obliczyć długość wektora. W naszych ćwiczeniach liczyliśmy ją raczej "na papierze" lub wykorzystując funkcje biblioteczne. W kontekście kodu, znajomość długości wektora może posłużyć np. do normalizacji (sprowadzenia wektora do jednostkowego o długości 1) lub do porównywania odległości. Na razie jednak wystarczy rozumieć, że jest to zastosowanie geometrii trójkąta prostokątnego – długość wektora to przeciwprostokątna trójkąta utworzonego z jego składowych.

Podsumowując: wektory pozwalają w łatwy sposób manipulować pozycjami obiektów. W tej części laboratorium nauczyliśmy się odróżniać wektor od punktu (wektor opisuje *zmianę* położenia), zobaczyliśmy jak dodawanie wektorów wpływa na ruch obiektu oraz jak skalowanie wektora zmienia wielkość ruchu. Te podstawy wektorów są niezwykle ważne – praktycznie każda akcja w grafice (ruch kamery, animacja obiektu, obliczenia fizyczne) opiera się na operacjach wektorowych.
